# Continuous optimization 

# Exercise 3

# Q1
data <- 
  read.table("/Users/Thierry/Documents/R/Data/Compstat/F5_2.txt",header=TRUE)
attach(data)


n=length(realdpi)
z<-realdpi#[seq(1,n,8)]
y<-realcons#[seq(1,n,8)]

plot(z,y,xlab="income",ylab="consumption")

reg<-lm(y ~z)
lines(z,reg$fitted.values,col='red',lwd=2)

plot(z,reg$residuals,xlab="income",ylab="residuals")
abline(h=0)

# Q2

LS <- function(theta,y,z) return(sum((y-(theta[1]+theta[2]*z^theta[3]))^2))
delta<-1
theta0<-c(reg$coefficients,1)
#theta0<-c(rnorm(2),1+runif(1))    # random initial solution
g<-LS(theta0,y,z)
t<-0
print(c(t,theta0,g,delta))
while(delta>1e-6){
  t<-t+1
  X <- y-(theta0[1]+theta0[2]*z^theta0[3])
  A<-cbind(rep(1,n),z^theta0[3],theta0[2]*log(z)*z^theta0[3])
  theta1<-theta0+solve(t(A)%*%A)%*%t(A)%*%X
  delta<-sqrt(sum((theta1-theta0)^2))/sqrt(sum((theta0)^2))
  g<-LS(theta1,y,z)
  print(c(t,theta1,g,delta))
  theta0<-theta1
}

yh<-theta0[1]+theta0[2]*z^theta0[3]
plot(z,y,xlab="income",ylab="consumption",pch=3)
lines(z,yh,col="red",lwd=2)

plot(z,y-yh,xlab="income",ylab="residuals")
abline(h=0)
points(z,reg$residuals,pch=3,col="red")

# Use optim
theta0<-c(reg$coefficients,1)
opt<-optim(theta0,fn=LS,method='BFGS',control=list(trace=1),y=y,z=z)

#  Exercise 4

# a -----------
transport<-read.table("/Users/Thierry/Documents/R/Data/Compstat/transportation.txt",
                      header=TRUE)

# b -----------
plot(log(transport))

# c -----------
reg <- lm(log(ValueAdd)~log(Capital)+log(Labor),data=transport)
beta0<-coef(reg)

# d -----------

# Log-likelihood function
loglik<- function(theta,x,y){
	n<-nrow(x)
	p<-ncol(x)
	beta<-theta[1:(p+1)]
	sigma<-theta[p+2]
	lambda<-theta[p+3]
	epsi<- y-beta[1] - x %*% beta[2:(p+1)]
	loglik<- -n*log(abs(sigma)) + (n/2)*log(2/pi) - 0.5*sum((epsi/sigma)^2) + 
	         sum(log(pnorm(-epsi*lambda/abs(sigma))))
	return(loglik)
}

sigma0=sqrt(mean(reg$res^2))
lambda0=1
theta0=c(beta0,sigma0,lambda0)

# Optimization using the BFGS algorithm
opt<-optim(theta0, loglik, gr = NULL,method = "BFGS", 
           hessian = FALSE,control=list(fnscale=-1),
           x=cbind(log(transport$Capital),log(transport$Labor)),
           y=log(transport$ValueAdd))

# e ---------------------
thetah<-opt$par
xx<-seq(1,3,0.005)
yy<-seq(0.1,0.5,0.0005)
nx=length(xx)
ny=length(yy)
z <- matrix(0,nrow=nx,ncol=ny)
for(i in 1:nx){
	for(j in 1:ny){
		z[i,j]=-loglik(c(xx[i],thetah[2:3],yy[j],thetah[5]),x=cbind(log(transport$Capital),log(transport$Labor)),y=log(transport$ValueAdd))
	}
}

contour(x=xx,y=yy,z,levels=c(-18,-15,-10,0,2,10,50,100,200,300,500,1000, 1500))
points(thetah[1],thetah[4],pch=3)
title(xlab=expression(beta[0]),ylab=expression(sigma))

# f ----------------------

theta0=c(3,beta0[2:3],0.5,10)
opt1<-optim(theta0, loglik, gr = NULL,method = "BFGS", hessian = FALSE,control=list(fnscale=-1),x=cbind(log(transport$Capital),log(transport$Labor)),y=log(transport$ValueAdd))

thetah<-opt1$par
xx<-seq(2.5,3.2,0.005)
yy<-seq(0.2,0.8,0.0005)
nx=length(xx)
ny=length(yy)
z <- matrix(0,nrow=nx,ncol=ny)
for(i in 1:nx){
	for(j in 1:ny){
		z[i,j]=-loglik(c(xx[i],thetah[2:3],yy[j],thetah[5]),x=cbind(log(transport$Capital),log(transport$Labor)),y=log(transport$ValueAdd))
	}
}
contour(x=xx,y=yy,z,levels=c(-17,-15,-10,-5,0,2,10,20,50,100,500,1000, 2000, 3000,5000,7000))
points(thetah[1],thetah[4],pch=20)
title(xlab=expression(beta[0]),ylab=expression(sigma))


